import sys, os, logging, string

import split_files
import streaming_extractor
import unpacker
import log
import PacketServer
import plesk_config
import osutil
import stacktrace
import daemon
import gzip_stream
from ReadLineAdapter import ReadLineAdapter


_logger = logging.getLogger("backup_extractor")


class BackupMimeExtractorHandler(PacketServer.PacketHandler):
    def __init__(self, request, client_address, server, se):
        self.se = se
        PacketServer.PacketHandler.__init__(self, request, client_address, server)

    def write_encoded_response(self, l):
        self.write_response("\0".join(l))

    def write_error(self, error):
        self.write_encoded_response(["ERR", error])

    def write_success(self, l):
        self.write_encoded_response(["OK"] + l)

    def decode_packet(self, packet):
        res = string.split(packet, "\0")
        return res[0], res[1:]

    def processPacket(self, packet):
        (command, args) = self.decode_packet(packet)

        _logger.debug(command + "(" + (", ".join(args)) + ")")

        if command == "untar":
            if len(args) != 2:
                self.write_error("'untar' command requires 2 arguments")
                return

            try:
                files = self.se.extract_part(args[0], unpacker.TarUnpacker(args[1]))
            except Exception, e:
                self.write_error("Runtime error during 'untar': " + unicode(e))
                return

            _logger.info("'untar' successfully completed. Files:")
            for f in files:
                _logger.debug(f)

            self.write_success(files)
            return

        if command == "db":
            if len(args) != 7:
                self.write_error("'db' command requires 7 arguments: cid, db name, db type, db user, db password, db host, db port")
                return

            try:
                self.se.extract_part(args[0], unpacker.DbUnpacker(args[1], args[2], args[3],
                                                                  args[4], args[5], args[6]))
            except Exception, e:
                self.write_error("Runtime error during 'db': " + unicode(e))
                return

            self.write_success([])
            return

        if command == "unpack":
            if len(args) != 2:
                self.write_error("'unpack' command requires 2 arguments")
                return
            try:
                files = self.se.extract_part(args[0], unpacker.PartUnpacker(args[1]))
            except Exception, e:
                self.write_error("Runtime error during 'unpack': " + unicode(e))
                return

            self.write_success([])
            return

        if command == "stop":
            self.write_success([])
            raise PacketServer.StopServer(None)

        self.write_error("Unknown command")


def usage():
    print "Usage: backup-extractor <session-path> [<first-backup-file>]"


def main():
    debug_mode = os.environ.has_key("DEBUG_BACKUP_MIME_EXTRACTOR")

    if len(sys.argv) != 2 and len(sys.argv) != 3:
        usage()
        sys.exit(1)

    session_dir = sys.argv[1].decode("UTF-8")
    backup_file = sys.argv[2].decode("UTF-8")

    log.initLog("backup-extractor", session_dir, debug_mode)
    _logger.info("backup-extractor started")

    try:
        if debug_mode:
            socket = "/tmp/mime-socket"
        else:
            socket = os.path.join(plesk_config.get("PRODUCT_ROOT_D"),
                                  "tmp",
                                  "backup-extractor." + str(os.getpid()))

        if os.path.exists(socket):
            try:
                os.unlink(socket)
            except OSError, e:
                sys.stderr.write("Unable to remove stale socket file: " + e.strerror)
                sys.exit(1)

        backupParts = split_files.findParts(backup_file)
        backup_cat_cmd = split_files.open(backupParts)

        if gzip_stream.isGzipped(backup_file):
            backup_stream = gzip_stream.GzipReadStream(backup_cat_cmd.stdout)
            backup_stream = ReadLineAdapter(backup_stream)
        else:
            backup_stream = backup_cat_cmd.stdout

        se = streaming_extractor.StreamingExtractor(backup_stream, session_dir)

        def backupMimeExtractorHandlerCreator(request, client_address, server):
            BackupMimeExtractorHandler(request, client_address, server, se)

        server = PacketServer.PacketServer(socket, backupMimeExtractorHandlerCreator, 0, 0755)

        sys.stdout.write(socket)

        if not debug_mode:
            daemon.daemon()
    except Exception, e:
        sys.stderr.write("Unable to start backup-mime-extractor: " + unicode(e.__class__)  + " " \
                         + unicode(e) + "\n" + stacktrace.stacktrace())
        sys.exit(1)

    try:
        try:
            server.serve_forever()
        finally:
            osutil.unlink_nothrow(socket)
    except PacketServer.StopServer:
        _logger.info("backup-extractor finished")
    except Exception, e:
        _logger.critical("Runtime error in backup-extractor: " + unicode(e.__class__) + " " \
                         + unicode(e) + "\n" + stacktrace.stacktrace())

if __name__ == '__main__':
    main()
